@@ -8,7 +8,7 @@ import sys
 
 import indexclient.client as client
 
-from gen3.utils import DEFAULT_BACKOFF_SETTINGS, raise_for_status
+from gen3.utils import DEFAULT_BACKOFF_SETTINGS, raise_for_status_and_print_error
 from gen3.auth import Gen3Auth
 
 logging = get_logger("__name__")
@@ -72,7 +72,7 @@ class Gen3Index:
 
         """
         response = self.client._get("_version")
-        raise_for_status(response)
+        raise_for_status_and_print_error(response)
         return response.json()
 
     @backoff.on_exception(backoff.expo, Exception, **DEFAULT_BACKOFF_SETTINGS)
@@ -83,7 +83,7 @@ class Gen3Index:
 
         """
         response = self.client._get("_stats")
-        raise_for_status(response)
+        raise_for_status_and_print_error(response)
         return response.json()
 
     @backoff.on_exception(backoff.expo, Exception, **DEFAULT_BACKOFF_SETTINGS)
@@ -100,7 +100,7 @@ class Gen3Index:
             url += f"?limit={limit}"
 
         response = self.client._get(url)
-        raise_for_status(response)
+        raise_for_status_and_print_error(response)
 
         records = response.json().get("records")
         all_records.extend(records)
@@ -121,7 +121,7 @@ class Gen3Index:
 
                 url = urllib.parse.urlunparse(url_parts)
                 response = self.client._get(url)
-                raise_for_status(response)
+                raise_for_status_and_print_error(response)
 
                 records = response.json().get("records")
                 all_records.extend(records)
@@ -150,7 +150,7 @@ class Gen3Index:
         query = urllib.parse.urlencode(params)
 
         response = self.client._get(url + "?" + query)
-        raise_for_status(response)
+        raise_for_status_and_print_error(response)
 
         return response.json().get("records")
 
@@ -168,7 +168,7 @@ class Gen3Index:
         url = f"{self.client.url}/index/{guid}"
         async with aiohttp.ClientSession() as session:
             async with session.get(url, ssl=_ssl) as response:
-                raise_for_status(response)
+                raise_for_status_and_print_error(response)
                 response = await response.json()
 
         return response
@@ -202,6 +202,34 @@ class Gen3Index:
 
         return response.get("records")
 
+    @backoff.on_exception(backoff.expo, Exception, **DEFAULT_BACKOFF_SETTINGS)
+    async def async_get_records_from_checksum(
+        self, checksum, checksum_type="md5", _ssl=None
+    ):
+        """
+        Asynchronous function to request records from indexd matching checksum.
+
+        Args:
+            checksum (str): indexd checksum to request
+            checksum_type (str): type of checksum, defaults to md5
+
+        Returns:
+            List[dict]: List of indexd records
+        """
+        all_records = []
+        params = {}
+
+        params["hash"] = f"{checksum_type}:{checksum}"
+
+        query = urllib.parse.urlencode(params)
+
+        url = f"{self.client.url}/index" + "?" + query
+        async with aiohttp.ClientSession() as session:
+            async with session.get(url, ssl=_ssl) as response:
+                response = await response.json()
+
+        return response.get("records")
+
     @backoff.on_exception(backoff.expo, Exception, **DEFAULT_BACKOFF_SETTINGS)
     def get(self, guid, dist_resolution=True):
         """
@@ -346,7 +374,7 @@ class Gen3Index:
 
         """
         response = self.client._get(f"/index/{guid}/versions")
-        raise_for_status(response)
+        raise_for_status_and_print_error(response)
         versions = response.json()
 
         return [r for _, r in versions.items()]
@@ -468,7 +496,7 @@ class Gen3Index:
                 ssl=_ssl,
                 auth=self.client.auth,
             ) as response:
-                raise_for_status(response)
+                raise_for_status_and_print_error(response)
                 response = await response.json()
 
         return response
@@ -494,7 +522,7 @@ class Gen3Index:
             auth=self.client.auth,
             data=client.json_dumps(json),
         )
-        raise_for_status(response)
+        raise_for_status_and_print_error(response)
         rec = response.json()
 
         return self.get_record(rec["did"])
@@ -567,7 +595,7 @@ class Gen3Index:
             data=client.json_dumps(json),
             auth=self.client.auth,
         )
-        raise_for_status(response)
+        raise_for_status_and_print_error(response)
         rec = response.json()
 
         if rec and "did" in rec:
@@ -631,7 +659,7 @@ class Gen3Index:
             auth=self.client.auth,
             data=client.json_dumps(json),
         )
-        raise_for_status(response)
+        raise_for_status_and_print_error(response)
         rec = response.json()
 
         return self.get_record(rec["did"])
@@ -722,7 +750,7 @@ class Gen3Index:
                 ssl=_ssl,
                 auth=self.client.auth,
             ) as response:
-                raise_for_status(response)
+                raise_for_status_and_print_error(response)
                 response = await response.json()
 
         return response
@@ -762,7 +790,7 @@ class Gen3Index:
             List[records]: indexd records with urls matching pattern
         """
         response = self.client._get(f"/_query/urls/q?include={pattern}")
-        raise_for_status(response)
+        raise_for_status_and_print_error(response)
         return response.json()
 
     @backoff.on_exception(backoff.expo, Exception, **DEFAULT_BACKOFF_SETTINGS)
@@ -780,7 +808,7 @@ class Gen3Index:
         async with aiohttp.ClientSession() as session:
             logging.debug(f"request: {url}")
             async with session.get(url, ssl=_ssl) as response:
-                raise_for_status(response)
+                raise_for_status_and_print_error(response)
                 response = await response.json()
 
         return response
